@@ -169,8 +169,9 @@ static int string_to_int(const char *s)
 	long l;
 	char *p;
 
+	errno = 0;
 	l = strtol(s, &p, 0);
-	if (*p || l == LONG_MIN || l == LONG_MAX || l < 0 || l > INT_MAX)
+	if (errno || *p || l == LONG_MIN || l == LONG_MAX || l < 0 || l > INT_MAX)
 		return -1;
 
 	return (int) l;
@@ -1089,7 +1090,7 @@ static int fs_ignored_type(struct libmnt_fs *fs)
 {
 	const char **ip, *type;
 
-	if (mnt_fs_is_netfs(fs) || mnt_fs_is_pseudofs(fs) || mnt_fs_is_swaparea(fs))
+	if (!mnt_fs_is_regularfs(fs))
 		return 1;
 
 	type = mnt_fs_get_fstype(fs);
@@ -1603,8 +1604,8 @@ static void parse_argv(int argc, char *argv[])
 
 	if (getenv("FSCK_FORCE_ALL_PARALLEL"))
 		force_all_parallel++;
-	if ((tmp = getenv("FSCK_MAX_INST")))
-	    max_running = atoi(tmp);
+	if (ul_strtos32(getenv("FSCK_MAX_INST"), &max_running, 10) != 0)
+		max_running = 0;
 }
 
 int main(int argc, char *argv[])
@@ -1632,6 +1633,8 @@ int main(int argc, char *argv[])
 	if (!notitle)
 		printf(UTIL_LINUX_VERSION);
 
+	signal(SIGCHLD, SIG_DFL);	/* clear any inherited settings */
+
 	load_fs_info();
 
 	fsck_path = xstrdup(path && *path ? path : FSCK_DEFAULT_PATH);
