@@ -165,7 +165,7 @@ static int sysfs_blkdev_enoent_redirect(struct path_cxt *pc, const char *path, i
 {
 	struct sysfs_blkdev *blk = ul_path_get_dialect(pc);
 
-	if (blk && blk->parent && strncmp(path, "queue/", 6) == 0) {
+	if (blk && blk->parent && path && strncmp(path, "queue/", 6) == 0) {
 		*dirfd = ul_path_get_dirfd(blk->parent);
 		if (*dirfd >= 0) {
 			DBG(CXT, ul_debugobj(pc, "%s redirected to parent", path));
@@ -210,9 +210,10 @@ int sysfs_blkdev_is_partition_dirent(DIR *dir, struct dirent *d, const char *par
 	    d->d_type != DT_UNKNOWN)
 		return 0;
 #endif
+	size_t len = 0;
+
 	if (parent_name) {
 		const char *p = parent_name;
-		size_t len;
 
 		/* /dev/sda --> "sda" */
 		if (*parent_name == '/') {
@@ -223,14 +224,15 @@ int sysfs_blkdev_is_partition_dirent(DIR *dir, struct dirent *d, const char *par
 		}
 
 		len = strlen(p);
-		if (strlen(d->d_name) <= len)
-			return 0;
+		if ((strlen(d->d_name) <= len) || (strncmp(p, d->d_name, len) != 0))
+			len = 0;
+	}
 
+	if (len > 0) {
 		/* partitions subdir name is
 		 *	"<parent>[:digit:]" or "<parent>p[:digit:]"
 		 */
-		return strncmp(p, d->d_name, len) == 0 &&
-		       ((*(d->d_name + len) == 'p' && isdigit(*(d->d_name + len + 1)))
+		return ((*(d->d_name + len) == 'p' && isdigit(*(d->d_name + len + 1)))
 			|| isdigit(*(d->d_name + len)));
 	}
 
@@ -1037,6 +1039,40 @@ int sysfs_devno_count_partitions(dev_t devno)
 	return n;
 }
 
+char *sysfs_chrdev_devno_to_devname(dev_t devno, char *buf, size_t bufsiz)
+{
+	char link[PATH_MAX];
+	struct path_cxt *pc;
+	char *name;
+	ssize_t	sz;
+
+	pc = ul_new_path(_PATH_SYS_DEVCHAR "/%u:%u", major(devno), minor(devno));
+	if (!pc)
+		return NULL;
+
+        /* read /sys/dev/char/<maj:min> link */
+	sz = ul_path_readlink(pc, link, sizeof(link), NULL);
+	ul_unref_path(pc);
+
+	if (sz < 0)
+		return NULL;
+
+	name = strrchr(link, '/');
+	if (!name)
+		return NULL;
+
+	name++;
+	sz = strlen(name);
+	if ((size_t) sz + 1 > bufsiz)
+		return NULL;
+
+	memcpy(buf, name, sz + 1);
+	sysfs_devname_sys_to_dev(buf);
+	return buf;
+
+}
+
+
 
 #ifdef TEST_PROGRAM_SYSFS
 #include <errno.h>
