@@ -30,6 +30,7 @@
 #include "mountP.h"
 #include "mangle.h"
 #include "pathnames.h"
+#include "strutils.h"
 
 struct libmnt_update {
 	char		*target;
@@ -762,15 +763,49 @@ static int update_modify_target(struct libmnt_update *upd, struct libmnt_lock *l
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB, 1);
 	if (tb) {
-		struct libmnt_fs *cur = mnt_table_find_target(tb,
-				mnt_fs_get_srcpath(upd->fs), MNT_ITER_BACKWARD);
-		if (cur) {
-			rc = mnt_fs_set_target(cur, mnt_fs_get_target(upd->fs));
-			if (!rc)
-				rc = update_table(upd, tb);
+		const char *upd_source = mnt_fs_get_srcpath(upd->fs);
+		const char *upd_target = mnt_fs_get_target(upd->fs);
+		struct libmnt_iter itr;
+		struct libmnt_fs *fs;
+		char *cn_target = mnt_resolve_path(upd_target, NULL);
+
+		if (!cn_target) {
+			rc = -ENOMEM;
+			goto done;
 		}
+
+		mnt_reset_iter(&itr, MNT_ITER_BACKWARD);
+		while (mnt_table_next_fs(tb, &itr, &fs) == 0) {
+			char *p;
+			const char *e;
+
+			e = startswith(mnt_fs_get_target(fs), upd_source);
+			if (!e || (*e && *e != '/'))
+				continue;
+			if (*e == '/')
+				e++;		/* remove extra '/' */
+
+			/* no subdirectory, replace entire path */
+			if (!*e)
+				rc = mnt_fs_set_target(fs, cn_target);
+
+			/* update start of the path, keep subdirectory */
+			else if (asprintf(&p, "%s/%s", cn_target, e) > 0) {
+				rc = mnt_fs_set_target(fs, p);
+				free(p);
+			} else
+				rc = -ENOMEM;
+
+			if (rc < 0)
+				break;
+		}
+
+		if (!rc)
+			rc = update_table(upd, tb);
+		free(cn_target);
 	}
 
+done:
 	if (lc)
 		mnt_unlock_file(lc);
 
