@@ -69,6 +69,7 @@ int main(int argc, char *argv[])
 {
 	struct libscols_table *tb;
 	size_t i;
+	const size_t timecellsz = sizeof(stringify_value(UINT_MAX));
 	struct timeval last;
 
 	scols_init_debug(0);
@@ -84,14 +85,15 @@ int main(int argc, char *argv[])
 		struct libscols_line *line;
 		struct timeval now;
 		int done = 0;
-		char *timecell = xmalloc( sizeof(stringify_value(UINT_MAX)) );
+		char *timecell = xmalloc( timecellsz );
 
 		line = add_line(tb, i);
 
 		/* Make a reference from cell data to the buffer, then we can
 		 * update cell data without any interaction with libsmartcols
 		 */
-		scols_line_refer_data(line, COL_TIME, timecell);
+		if (scols_line_refer_data(line, COL_TIME, timecell) != 0)
+			err(EXIT_FAILURE, "failed to add data to table");
 
 		do {
 			double diff;
@@ -105,11 +107,19 @@ int main(int argc, char *argv[])
 				xusleep(100000);
 
 			/* update "TIME" cell data */
-			sprintf(timecell, "%f [%3d%%]", diff,
+			snprintf(timecell, timecellsz, "%f [%3d%%]", diff,
 				done ? 100 : (int)(diff / (TIME_PERIOD / 100.0)));
 
-			/* don't print line separator when in progress */
-			scols_table_enable_nolinesep(tb, !done);
+			/* Note that libsmartcols don't print \n for last line
+			 * in the table, but if you print a line somewhere in
+			 * the midle of the table you need
+			 *
+			 *    scols_table_enable_nolinesep(tb, !done);
+			 *
+			 * to disable line breaks. In this example it's
+			 * unnecessary as we print the latest line only.
+			 */
+
 			/* print the line */
 			scols_table_print_range(tb, line, NULL);
 
@@ -118,7 +128,8 @@ int main(int argc, char *argv[])
 				fflush(scols_table_get_stream(tb));
 				/* move to the begin of the line */
 				fputc('\r', scols_table_get_stream(tb));
-			}
+			} else
+				fputc('\n', scols_table_get_stream(tb));
 		} while (!done);
 
 		last = now;
