@@ -47,6 +47,7 @@ struct libscols_line *scols_new_line(void)
 	INIT_LIST_HEAD(&ln->ln_lines);
 	INIT_LIST_HEAD(&ln->ln_children);
 	INIT_LIST_HEAD(&ln->ln_branch);
+	INIT_LIST_HEAD(&ln->ln_groups);
 	return ln;
 }
 
@@ -75,6 +76,8 @@ void scols_unref_line(struct libscols_line *ln)
 		DBG(CELL, ul_debugobj(ln, "dealloc"));
 		list_del(&ln->ln_lines);
 		list_del(&ln->ln_children);
+		list_del(&ln->ln_groups);
+		scols_unref_group(ln->group);
 		scols_line_free_cells(ln);
 		free(ln->color);
 		free(ln);
@@ -217,7 +220,7 @@ int scols_line_remove_child(struct libscols_line *ln, struct libscols_line *chil
 	if (!ln || !child)
 		return -EINVAL;
 
-	DBG(LINE, ul_debugobj(ln, "remove child %p", child));
+	DBG(LINE, ul_debugobj(ln, "remove child"));
 
 	list_del_init(&child->ln_children);
 	child->parent = NULL;
@@ -241,7 +244,7 @@ int scols_line_add_child(struct libscols_line *ln, struct libscols_line *child)
 	if (!ln || !child)
 		return -EINVAL;
 
-	DBG(LINE, ul_debugobj(ln, "add child %p", child));
+	DBG(LINE, ul_debugobj(ln, "add child"));
 	scols_ref_line(child);
 	scols_ref_line(ln);
 
@@ -309,6 +312,26 @@ int scols_line_next_child(struct libscols_line *ln,
 	return rc;
 }
 
+/* private API */
+int scols_line_next_group_child(struct libscols_line *ln,
+			  struct libscols_iter *itr,
+			  struct libscols_line **chld)
+{
+	int rc = 1;
+
+	if (!ln || !itr || !chld || !ln->group)
+		return -EINVAL;
+	*chld = NULL;
+
+	if (!itr->head)
+		SCOLS_ITER_INIT(itr, &ln->group->gr_children);
+	if (itr->p != itr->head) {
+		SCOLS_ITER_ITERATE(itr, *chld, struct libscols_line, ln_children);
+		rc = 0;
+	}
+
+	return rc;
+}
 
 /**
  * scols_line_is_ancestor:
@@ -441,6 +464,26 @@ int scols_line_set_column_data(struct libscols_line *ln,
 	return scols_line_set_data(ln, cl->seqnum, data);
 }
 
+/**
+ * scols_line_get_column_data:
+ * @ln: a pointer to a struct libscols_line instance
+ * @cl: column, whose data is to be get
+ *
+ * See also scols_cell_get_data()
+ *
+ * Returns: cell data or NULL.
+ *
+ * Since: 2.38
+ */
+const char *scols_line_get_column_data(struct libscols_line *ln,
+			       struct libscols_column *cl)
+{
+	struct libscols_cell *cell = scols_line_get_column_cell(ln, cl);
+
+	return cell ? scols_cell_get_data(cell) : NULL;
+}
+
+
 /**
  * scols_line_refer_data:
  * @ln: a pointer to a struct libscols_line instance
@@ -503,7 +546,7 @@ struct libscols_line *scols_copy_line(const struct libscols_line *ln)
 	ret->ncells   = ln->ncells;
 	ret->seqnum   = ln->seqnum;
 
-	DBG(LINE, ul_debugobj(ln, "copy to %p", ret));
+	DBG(LINE, ul_debugobj(ln, "copy"));
 
 	for (i = 0; i < ret->ncells; ++i) {
 		if (scols_cell_copy_content(&ret->cells[i], &ln->cells[i]))
