@@ -55,8 +55,9 @@
 #include "strutils.h"
 #include "optutils.h"
 #include "pathnames.h"
+#include "fileutils.h"
 #include "logindefs.h"
-#include "procutils.h"
+#include "procfs.h"
 #include "timeutils.h"
 
 /*
@@ -225,7 +226,7 @@ static const struct lslogins_coldesc coldescs[] =
 {
 	[COL_USER]          = { "USER",		N_("user name"), N_("Username"), 0.1, SCOLS_FL_NOEXTREMES },
 	[COL_UID]           = { "UID",		N_("user ID"), "UID", 1, SCOLS_FL_RIGHT},
-	[COL_PWDEMPTY]      = { "PWD-EMPTY",	N_("password not required"), N_("Password not required"), 1, SCOLS_FL_RIGHT },
+	[COL_PWDEMPTY]      = { "PWD-EMPTY",	N_("password not defined"), N_("Password not required (empty)"), 1, SCOLS_FL_RIGHT },
 	[COL_PWDDENY]       = { "PWD-DENY",	N_("login by password disabled"), N_("Login by password disabled"), 1, SCOLS_FL_RIGHT },
 	[COL_PWDLOCK]       = { "PWD-LOCK",	N_("password defined, but locked"), N_("Password is locked"), 1, SCOLS_FL_RIGHT },
 	[COL_PWDMETHOD]     = { "PWD-METHOD",   N_("password encryption method"), N_("Password encryption method"), 0.1 },
@@ -280,6 +281,7 @@ struct lslogins_control {
 	unsigned int selinux_enabled : 1,
 		     fail_on_unknown : 1,		/* fail if user does not exist */
 		     ulist_on : 1,
+		     shellvar : 1,
 		     noheadings : 1,
 		     notrunc : 1;
 };
@@ -488,7 +490,7 @@ static int parse_utmpx(const char *path, size_t *nrecords, struct utmpx **record
 
 	/* optimize allocation according to file size, the realloc() below is
 	 * just fallback only */
-	if (stat(path, &st) == 0 && (size_t) st.st_size > sizeof(struct utmpx)) {
+	if (stat(path, &st) == 0 && (size_t) st.st_size >= sizeof(struct utmpx)) {
 		imax = st.st_size / sizeof(struct utmpx);
 		ary = xmalloc(imax * sizeof(struct utmpx));
 	}
@@ -562,9 +564,6 @@ static int get_sgroups(gid_t **list, size_t *len, struct passwd *pwd)
 
 	*list = xcalloc(1, ngroups * sizeof(gid_t));
 
-fprintf(stderr, "KZAK>>> alloc '%p' for %s\n", *list, pwd->pw_name);
-
-
 	/* now for the actual list of GIDs */
 	if (-1 == getgrouplist(pwd->pw_name, pwd->pw_gid, *list, &ngroups))
 		return -1;
@@ -587,16 +586,20 @@ fprintf(stderr, "KZAK>>> alloc '%p' for %s\n", *list, pwd->pw_name);
 #ifdef __linux__
 static int get_nprocs(const uid_t uid)
 {
+	DIR *dir;
+	struct dirent *d;
 	int nprocs = 0;
-	pid_t pid;
-	struct proc_processes *proc = proc_open_processes();
 
-	proc_processes_filter_by_uid(proc, uid);
+	dir = opendir(_PATH_PROC);
+	if (!dir)
+		return 0;
 
-	while (!proc_next_pid(proc, &pid))
-		++nprocs;
+	while ((d = xreaddir(dir))) {
+		if (procfs_dirent_match_uid(dir, d, uid))
+			++nprocs;
+	}
 
-	proc_close_processes(proc);
+	closedir(dir);
 	return nprocs;
 }
 #endif
@@ -820,23 +823,42 @@ static struct lslogins_user *get_user_info(struct lslogins_control *ctl, const c
 			break;
 		case COL_PWDEMPTY:
 			if (shadow) {
-				if (!*shadow->sp_pwdp) /* '\0' */
+				const char *p = shadow->sp_pwdp;
+
+				while (p && (*p == '!' || *p == '*'))
+					p++;
+
+				if (!p || !*p)
 					user->pwd_empty = STATUS_TRUE;
 			} else
 				user->pwd_empty = STATUS_UNKNOWN;
 			break;
 		case COL_PWDDENY:
 			if (shadow) {
-				if ((*shadow->sp_pwdp == '!' ||
-				     *shadow->sp_pwdp == '*') &&
-				    !valid_pwd(shadow->sp_pwdp + 1))
+				const char *p = shadow->sp_pwdp;
+
+				while (p && (*p == '!' || *p == '*'))
+					p++;
+
+				if (p && *p && p != shadow->sp_pwdp && !valid_pwd(p))
 					user->pwd_deny = STATUS_TRUE;
 			} else
 				user->pwd_deny = STATUS_UNKNOWN;
 			break;
 		case COL_PWDLOCK:
 			if (shadow) {
-				if (*shadow->sp_pwdp == '!' && valid_pwd(shadow->sp_pwdp + 1))
+				const char *p = shadow->sp_pwdp;
+				int i = 0;
+
+				/* 'passwd --lock' uses two exclamation marks,
+				 * shadow(5) describes the lock as "field which
+				 * starts with an exclamation mark". Let's
+				 * support more '!' ...
+				 */
+				while (p && *p == '!')
+					p++, i++;
+
+				if (i != 0 && (!*p || valid_pwd(p)))
 					user->pwd_lock = STATUS_TRUE;
 			} else
 				user->pwd_lock = STATUS_UNKNOWN;
@@ -845,7 +867,7 @@ static struct lslogins_user *get_user_info(struct lslogins_control *ctl, const c
 			if (shadow) {
 				const char *p = shadow->sp_pwdp;
 
-				if (*p == '!' || *p == '*')
+				while (p && (*p == '!' || *p == '*'))
 					p++;
 				user->pwd_method = get_pwd_method(p, NULL, NULL);
 			} else
@@ -991,6 +1013,9 @@ static void free_ctl(struct lslogins_control *ctl)
 {
 	size_t n = 0;
 
+	if (!ctl)
+		return;
+
 	free(ctl->wtmp);
 	free(ctl->btmp);
 
@@ -1070,6 +1095,8 @@ static struct libscols_table *setup_table(struct lslogins_control *ctl)
 		err(EXIT_FAILURE, _("failed to allocate output table"));
 	if (ctl->noheadings)
 		scols_table_enable_noheadings(table, 1);
+	if (ctl->shellvar)
+		scols_table_enable_shellvar(table, 1);
 
 	switch(outmode) {
 	case OUT_COLON:
@@ -1418,6 +1445,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -s, --system-accs        display system accounts\n"), out);
 	fputs(_("     --time-format=<type> display dates in short, full or iso format\n"), out);
 	fputs(_(" -u, --user-accs          display user accounts\n"), out);
+	fputs(_(" -y, --shell              use column names to be usable as shell variable identifiers\n"), out);
 	fputs(_(" -Z, --context            display SELinux contexts\n"), out);
 	fputs(_(" -z, --print0             delimit user entries with a nul character\n"), out);
 	fputs(_("     --wtmp-file <path>   set an alternate path for wtmp\n"), out);
@@ -1458,6 +1486,7 @@ int main(int argc, char *argv[])
 		{ "acc-expiration", no_argument,	0, 'a' },
 		{ "colon-separate", no_argument,	0, 'c' },
 		{ "export",         no_argument,	0, 'e' },
+		{ "shell",          no_argument,        0, 'y' },
 		{ "failed",         no_argument,	0, 'f' },
 		{ "groups",         required_argument,	0, 'g' },
 		{ "help",           no_argument,	0, 'h' },
@@ -1507,7 +1536,7 @@ int main(int argc, char *argv[])
 	add_column(columns, ncolumns++, COL_UID);
 	add_column(columns, ncolumns++, COL_USER);
 
-	while ((c = getopt_long(argc, argv, "acefGg:hLl:no:prsuVzZ",
+	while ((c = getopt_long(argc, argv, "acefGg:hLl:no:prsuVyzZ",
 				longopts, NULL)) != -1) {
 
 		err_exclusive_options(c, longopts, excl, excl_st);
@@ -1583,6 +1612,9 @@ int main(int argc, char *argv[])
 			add_column(columns, ncolumns++, COL_HUSH_STATUS);
 			add_column(columns, ncolumns++, COL_PWDMETHOD);
 			break;
+		case 'y':
+			ctl->shellvar = 1;
+			break;
 		case 'z':
 			outmode = OUT_NUL;
 			break;
