@@ -14,6 +14,8 @@
 #include "c.h"
 #include "strutils.h"
 #include "xalloc.h"
+#include "pathnames.h"
+#include "match.h"
 
 #include "findmnt.h"
 
@@ -35,7 +37,7 @@ struct verify_context {
 static void __attribute__ ((__format__ (__printf__, 3, 0)))
 	verify_mesg(struct verify_context *vfy, char type, const char *fmt, va_list ap)
 {
-	if (!vfy->target_printed) {
+	if (!vfy->target_printed && vfy->fs) {
 		fprintf(stdout, "%s\n", mnt_fs_get_target(vfy->fs));
 		vfy->target_printed = 1;
 	}
@@ -283,7 +285,7 @@ static int is_supported_filesystem(struct verify_context *vfy, const char *name)
 		return 0;
 
 	for (n = 0; n < vfy->fs_num; n++ ) {
-		if (strcmp(vfy->fs_ary[n], name) == 0)
+		if (match_fstype(vfy->fs_ary[n], name))
 			return 1;
 	}
 
@@ -436,25 +438,30 @@ static int verify_fstype(struct verify_context *vfy)
 			isauto = 1;
 		else if (strcmp(type, "swap") == 0)
 			isswap = 1;
-		else if (strcmp(type, "xfs") == 0)
+		else if (strcmp(type, "xfs") == 0 || strcmp(type, "btrfs") == 0)
 			vfy->no_fsck = 1;
 
 		if (!isswap && !isauto && !none && !is_supported_filesystem(vfy, type))
 			verify_warn(vfy, _("%s seems unsupported by the current kernel"), type);
 	}
+
+	errno = 0;
 	realtype = mnt_get_fstype(src, &ambi, cache);
 
 	if (!realtype) {
+		const char *reson = errno ? strerror(errno) : _("reason unknown");
+
 		if (isauto)
-			verify_err(vfy, _("cannot detect on-disk filesystem type"));
+			verify_err(vfy, _("cannot detect on-disk filesystem type (%s)"), reson);
 		else
-			verify_warn(vfy, _("cannot detect on-disk filesystem type"));
+			verify_warn(vfy, _("cannot detect on-disk filesystem type (%s)"), reson);
 		goto done;
 	}
 
 	if (realtype) {
 		isswap = strcmp(realtype, "swap") == 0;
-		vfy->no_fsck = strcmp(realtype, "xfs") == 0;
+		vfy->no_fsck = strcmp(realtype, "xfs") == 0
+				|| strcmp(realtype, "btrfs") == 0;
 
 		if (type && !isauto && strcmp(type, realtype) != 0) {
 			verify_err(vfy, _("%s does not match with on-disk %s"), type, realtype);
@@ -545,6 +552,19 @@ int verify_table(struct libmnt_table *tb)
 		flags |= FL_NOSWAPMATCH;
 	}
 
+#ifdef USE_SYSTEMD
+	{
+		struct stat a, b;
+
+		if (stat(_PATH_SD_UNITSLOAD, &a) == 0 &&
+		    stat(_PATH_MNTTAB, &b) == 0 &&
+		    cmp_stat_mtime(&a, &b, <))
+			verify_warn(&vfy, _(
+	"your fstab has been modified, but systemd still uses the old version;\n"
+	"       use 'systemctl daemon-reload' to reload"));
+	}
+#endif
+
 done:
 	mnt_free_iter(itr);
 
