@@ -118,6 +118,7 @@ struct logger_ctl {
 	pid_t pid;			/* zero when unwanted */
 	char *hdr;			/* the syslog header (based on protocol) */
 	char const *tag;
+	char *login;
 	char *msgid;
 	char *unix_socket;		/* -u <path> or default to _PATH_DEVLOG */
 	char *server;
@@ -654,7 +655,7 @@ static char *strdup_structured_data_list(struct list_head *ls)
 
 		if (!one)
 			continue;
-		res = strappend(tmp, one);
+		res = strconcat(tmp, one);
 		free(tmp);
 		free(one);
 	}
@@ -672,7 +673,7 @@ static char *get_structured_data_string(struct logger_ctl *ctl)
 		usr = strdup_structured_data_list(&ctl->user_sds);
 
 	if (sys && usr) {
-		res = strappend(sys, usr);
+		res = strconcat(sys, usr);
 		free(sys);
 		free(usr);
 	} else
@@ -683,13 +684,37 @@ static char *get_structured_data_string(struct logger_ctl *ctl)
 
 static int valid_structured_data_param(const char *str)
 {
+	char *s;
 	char *eq  = strchr(str, '='),
 	     *qm1 = strchr(str, '"'),
-	     *qm2 = qm1 ? strchr(qm1 + 1, '"') : NULL;
+	     *qm2 = qm1 ? ul_strchr_escaped(qm1 + 1, '"') : NULL;
 
-	if (!eq || !qm1 || !qm2)		/* something is missing */
+	/* something is missing */
+	if (!eq || !qm1 || !qm2)
 		return 0;
 
+	/* ']' need to be escaped */
+	for (s = qm1 + 1; s && *s; ) {
+		char *p = strchr(s, ']');
+		if (!p)
+			break;
+		if (p > qm2 || p == ul_strchr_escaped(s, ']'))
+			return 0;
+		s = p + 1;
+	}
+
+	/* '\' is allowed only before '[]"\' chars */
+	for (s = qm1 + 1; s && *s; ) {
+		char *p = strchr(s, '\\');
+		if (!p)
+			break;
+		if (!strchr("[]\"\\", *(p + 1)))
+			return 0;
+		s = p + 1;
+		if (*s == '\\')
+			s++;
+	}
+
 	/* foo="bar" */
 	return eq > str && eq < qm1 && eq + 1 == qm1 && qm1 < qm2 && *(qm2 + 1) == '\0';
 }
@@ -917,7 +942,7 @@ static void logger_open(struct logger_ctl *ctl)
 		ctl->syslogfp = ctl->server ? syslog_rfc5424_header :
 					      syslog_local_header;
 	if (!ctl->tag)
-		ctl->tag = xgetlogin();
+		ctl->tag = ctl->login = xgetlogin();
 	if (!ctl->tag)
 		ctl->tag = "<someone>";
 
@@ -973,11 +998,9 @@ static void logger_stdin(struct logger_ctl *ctl)
 	 * update header timestamps and to reflect possible priority changes.
 	 * The initial header is generated by logger_open().
 	 */
-	int has_header = 1;
 	int default_priority = ctl->pri;
 	int last_pri = default_priority;
-	size_t max_usrmsg_size = ctl->max_message_size - strlen(ctl->hdr);
-	char *const buf = xmalloc(max_usrmsg_size + 2 + 2);
+	char *buf = xmalloc(ctl->max_message_size + 2 + 2);
 	int pri;
 	int c;
 	size_t i;
@@ -997,34 +1020,28 @@ static void logger_stdin(struct logger_ctl *ctl)
 			if (c == '>' && 0 <= pri && pri <= 191) {
 				/* valid RFC PRI values */
 				i = 0;
-				if (pri < 8)	/* kern facility is forbidden */
-					pri |= 8;
+				if ((pri & LOG_FACMASK) == 0)
+					pri |= (default_priority & LOG_FACMASK);
 				ctl->pri = pri;
 			} else
 				ctl->pri = default_priority;
 
 			if (ctl->pri != last_pri) {
-				has_header = 0;
-				max_usrmsg_size =
-				    ctl->max_message_size - strlen(ctl->hdr);
+				generate_syslog_header(ctl);
 				last_pri = ctl->pri;
 			}
 			if (c != EOF && c != '\n')
 				c = getchar();
 		}
 
-		while (c != EOF && c != '\n' && i < max_usrmsg_size) {
+		while (c != EOF && c != '\n' && i < ctl->max_message_size) {
 			buf[i++] = c;
 			c = getchar();
 		}
 		buf[i] = '\0';
 
-		if (i > 0 || !ctl->skip_empty_lines) {
-			if (!has_header)
-				generate_syslog_header(ctl);
+		if (i > 0 || !ctl->skip_empty_lines)
 			write_output(ctl, buf);
-			has_header = 0;
-		}
 
 		if (c == '\n')	/* discard line terminator */
 			c = getchar();
@@ -1038,6 +1055,7 @@ static void logger_close(const struct logger_ctl *ctl)
 	if (ctl->fd != -1 && close(ctl->fd) != 0)
 		err(EXIT_FAILURE, _("close failed"));
 	free(ctl->hdr);
+	free(ctl->login);
 }
 
 static void __attribute__((__noreturn__)) usage(void)
