@@ -273,7 +273,7 @@ static int blkzone_report(struct blkzone_control *ctl)
 		for (i = 0; i < zi->nr_zones; i++) {
 /*
  * blk_zone_report hasn't been packed since https://github.com/torvalds/linux/commit/b3e7e7d2d668de0102264302a4d10dd9d4438a42
- * was merged. See https://github.com/karelzak/util-linux/issues/1083
+ * was merged. See https://github.com/util-linux/util-linux/issues/1083
  */
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Waddress-of-packed-member"
@@ -298,7 +298,7 @@ static int blkzone_report(struct blkzone_control *ctl)
 
 			if (only_capacity_sum) {
 				capacity_sum += cap;
-			} else {
+			} else if (has_zone_capacity(zi)) {
 				printf(_("  start: 0x%09"PRIx64", len 0x%06"PRIx64
 					", cap 0x%06"PRIx64", wptr 0x%06"PRIx64
 					" reset:%u non-seq:%u, zcond:%2u(%s) [type: %u(%s)]\n"),
@@ -306,6 +306,14 @@ static int blkzone_report(struct blkzone_control *ctl)
 					entry->reset, entry->non_seq,
 					cond, condition_str[cond & (ARRAY_SIZE(condition_str) - 1)],
 					type, type_text[type]);
+			} else {
+				printf(_("  start: 0x%09"PRIx64", len 0x%06"PRIx64
+					", wptr 0x%06"PRIx64
+					" reset:%u non-seq:%u, zcond:%2u(%s) [type: %u(%s)]\n"),
+					start, len, (type == 0x1) ? 0 : wp - start,
+					entry->reset, entry->non_seq,
+					cond, condition_str[cond & (ARRAY_SIZE(condition_str) - 1)],
+					type, type_text[type]);
 			}
 
 			nr_zones--;
