@@ -77,7 +77,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c = getopt_long(argc, argv, "hfuV", longopts, NULL)) != -1) {
 
@@ -144,8 +144,7 @@ int main(int argc, char **argv)
 
 	rc = EXIT_SUCCESS;
 done:
-	if (fd >= 0)
-		close(fd);
+	close(fd);
 	return rc;
 }
 
