@@ -25,6 +25,8 @@
 #include <ctype.h>
 #include <assert.h>
 #include <unistd.h>
+#include <sys/time.h>
+#include <sys/types.h>
 #include <sys/resource.h>
 
 #include <libsmartcols.h>
@@ -141,15 +143,19 @@ static int ncolumns;
 static pid_t pid; /* calling process (default) */
 static int verbose;
 
-#ifndef HAVE_PRLIMIT
+#ifdef HAVE_SYS_SYSCALL_H
 # include <sys/syscall.h>
+# if defined(SYS_prlimit64)
+#  ifndef HAVE_PRLIMIT
 static int prlimit(pid_t p, int resource,
 		   const struct rlimit *new_limit,
 		   struct rlimit *old_limit)
 {
 	return syscall(SYS_prlimit64, p, resource, new_limit, old_limit);
 }
-#endif
+#  endif /* !HAVE_PRLIMIT */
+# endif /* SYS_prlimit64 */
+#endif /* HAVE_SYS_SYSCALL_H */
 
 static void __attribute__((__noreturn__)) usage(void)
 {
@@ -159,14 +165,14 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(USAGE_HEADER, out);
 
 	fprintf(out,
-		_(" %s [options] [-p PID]\n"), program_invocation_short_name);
+		_(" %s [options] [--<resource>=<limit>] [-p PID]\n"), program_invocation_short_name);
 	fprintf(out,
-		_(" %s [options] COMMAND\n"), program_invocation_short_name);
+		_(" %s [options] [--<resource>=<limit>] COMMAND\n"), program_invocation_short_name);
 
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("Show or change the resource limits of a process.\n"), out);
 
-	fputs(_("\nGeneral Options:\n"), out);
+	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -p, --pid <pid>        process id\n"
 		" -o, --output <list>    define which output columns to use\n"
 		"     --noheadings       don't print headings\n"
@@ -175,7 +181,7 @@ static void __attribute__((__noreturn__)) usage(void)
 		), out);
 	printf(USAGE_HELP_OPTIONS(24));
 
-	fputs(_("\nResources Options:\n"), out);
+	fputs(_("\nResources:\n"), out);
 	fputs(_(" -c, --core             maximum size of core files created\n"
 		" -d, --data             maximum size of a process's data segment\n"
 		" -e, --nice             maximum nice priority allowed to raise\n"
@@ -194,6 +200,11 @@ static void __attribute__((__noreturn__)) usage(void)
 		" -y, --rttime           CPU time in microseconds a process scheduled\n"
 		"                        under real-time scheduling\n"), out);
 
+	fputs(USAGE_ARGUMENTS, out);
+	fputs(_(
+		" <limit> is defined as a range soft:hard, soft:, :hard or a value to\n"
+		"         define both limits (e.g. -e=0:10 -r=:10).\n"), out);
+
 	fputs(USAGE_COLUMNS, out);
 	for (i = 0; i < ARRAY_SIZE(infos); i++)
 		fprintf(out, " %11s  %s\n", infos[i].name, _(infos[i].help));
@@ -233,10 +244,12 @@ static void add_scols_line(struct libscols_table *table, struct prlimit *l)
 
 		switch (get_column_id(i)) {
 		case COL_RES:
-			str = xstrdup(l->desc->name);
+			if (l->desc->name)
+				str = xstrdup(l->desc->name);
 			break;
 		case COL_HELP:
-			str = xstrdup(l->desc->help);
+			if (l->desc->help)
+				str = xstrdup(_(l->desc->help));
 			break;
 		case COL_SOFT:
 			if (l->rlim.rlim_cur == RLIM_INFINITY)
@@ -251,7 +264,8 @@ static void add_scols_line(struct libscols_table *table, struct prlimit *l)
 				xasprintf(&str, "%llu", (unsigned long long) l->rlim.rlim_max);
 			break;
 		case COL_UNITS:
-			str = l->desc->unit ? xstrdup(_(l->desc->unit)) : NULL;
+			if (l->desc->unit)
+				str = xstrdup(_(l->desc->unit));
 			break;
 		default:
 			break;
@@ -397,7 +411,9 @@ static int get_range(char *str, rlim_t *soft, rlim_t *hard, int *found)
 		*found |= PRLIMIT_SOFT | PRLIMIT_HARD;
 		return 0;
 
-	} else if (*str == ':') {			/* <:hard> */
+	}
+
+	if (*str == ':') {			/* <:hard> */
 		str++;
 
 		if (strcmp(str, INFINITY_STR) != 0) {
@@ -448,9 +464,12 @@ static int get_range(char *str, rlim_t *soft, rlim_t *hard, int *found)
 
 static int parse_prlim(struct rlimit *lim, char *ops, size_t id)
 {
-	rlim_t soft, hard;
+	rlim_t soft = 0, hard = 0;
 	int found = 0;
 
+	if (ops && *ops == '=')
+		ops++;
+
 	if (get_range(ops, &soft, &hard, &found))
 		errx(EXIT_FAILURE, _("failed to parse %s limit"),
 		     prlimit_desc[id].name);
