@@ -38,6 +38,7 @@ static void __attribute__((__noreturn__)) usage(void)
 
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -c, --ctty     set the controlling terminal to the current one\n"), out);
+	fputs(_(" -f, --fork     always fork\n"), out);
 	fputs(_(" -w, --wait     wait program to exit, and use the same return\n"), out);
 
 	printf(USAGE_HELP_OPTIONS(16));
@@ -48,13 +49,14 @@ static void __attribute__((__noreturn__)) usage(void)
 
 int main(int argc, char **argv)
 {
-	int ch;
+	int ch, forcefork = 0;
 	int ctty = 0;
 	pid_t pid;
 	int status = 0;
 
 	static const struct option longopts[] = {
 		{"ctty", no_argument, NULL, 'c'},
+		{"fork", no_argument, NULL, 'f'},
 		{"wait", no_argument, NULL, 'w'},
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
@@ -64,21 +66,24 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
-	while ((ch = getopt_long(argc, argv, "+Vhcw", longopts, NULL)) != -1)
+	while ((ch = getopt_long(argc, argv, "+Vhcfw", longopts, NULL)) != -1)
 		switch (ch) {
-		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
 		case 'c':
 			ctty=1;
 			break;
+		case 'f':
+			forcefork = 1;
+			break;
 		case 'w':
 			status = 1;
 			break;
+
 		case 'h':
 			usage();
+		case 'V':
+			print_version(EXIT_SUCCESS);
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
@@ -88,7 +93,7 @@ int main(int argc, char **argv)
 		errtryhelp(EXIT_FAILURE);
 	}
 
-	if (getpgrp() == getpid()) {
+	if (forcefork || getpgrp() == getpid()) {
 		pid = fork();
 		switch (pid) {
 		case -1:
@@ -114,5 +119,5 @@ int main(int argc, char **argv)
 	if (ctty && ioctl(STDIN_FILENO, TIOCSCTTY, 1))
 		err(EXIT_FAILURE, _("failed to set the controlling terminal"));
 	execvp(argv[optind], argv + optind);
-	err(EXIT_FAILURE, _("failed to execute %s"), argv[optind]);
+	errexec(argv[optind]);
 }
