@@ -172,7 +172,7 @@ static void search_utmp(struct write_control *ctl)
 		if (strncmp(ctl->dst_login, u->ut_user, sizeof(u->ut_user)) != 0)
 			continue;
 		num_ttys++;
-		sprintf(path, "/dev/%s", u->ut_line);
+		snprintf(path, sizeof(path), "/dev/%s", u->ut_line);
 		if (check_tty(path, &tty_writeable, &tty_atime, 0))
 			/* bad term? skip */
 			continue;
@@ -275,7 +275,7 @@ static void do_write(const struct write_control *ctl)
 	tm = localtime(&now);
 	/* print greeting */
 	printf("\r\n\a\a\a");
-	if (strcmp(login, pwuid))
+	if (strcmp(login, pwuid) != 0)
 		printf(_("Message from %s@%s (as %s) on %s at %02d:%02d ..."),
 		       login, host, pwuid, ctl->src_tty_name,
 		       tm->tm_hour, tm->tm_min);
