@@ -317,15 +317,14 @@ static void getwinsize(void)
 	struct winsize winsz;
 	int badioctl;
 #endif
-	char *p;
-
 	if (initialized == 0) {
-		if ((p = getenv("LINES")) != NULL && *p != '\0')
-			if ((envlines = atoi(p)) < 0)
-				envlines = 0;
-		if ((p = getenv("COLUMNS")) != NULL && *p != '\0')
-			if ((envcols = atoi(p)) < 0)
-				envcols = 0;
+		uint32_t tmp = 0;
+
+		if (ul_strtou32(getenv("LINES"), &tmp, 10) == 0)
+			envlines = tmp;
+		if (ul_strtou32(getenv("COLUMNS"), &tmp, 10) == 0)
+			envcols = tmp;
+
 		/* terminfo values. */
 		if (tinfostat != 1 || columns == 0)
 			defcols = 24;
@@ -453,7 +452,7 @@ static char *endline_for_mb(unsigned col, char *s)
  ended:
 	*end = L'\0';
 	p = wbuf;
-	if ((pos = wcstombs(NULL, p, READBUF)) == (size_t)-1)
+	if ((pos = wcstombs(NULL, p, 0)) == (size_t)-1)
 		return s + 1;
 	return s + pos;
 }
@@ -598,11 +597,8 @@ static int getcount(char *cmdstr)
 	}
 	if (buf[0] == '-' && buf[1] == '\0') {
 		i = -1;
-	} else {
-		if (*buf == '+')
-			i = atoi(buf + 1);
-		else
-			i = atoi(buf);
+	} else if (ul_strtos32(*buf == '+' ? buf + 1 : buf, &i, 10) != 0) {
+		i = -1;
 	}
 	free(buf);
 	return i;
@@ -987,42 +983,43 @@ static void pgfile(FILE *f, const char *name)
 					*b = '\0';
 					dline = pagelen;
 					break;
-				} else {
-					if (nobuf)
-						fseeko(f, fpos, SEEK_SET);
-					canjump = 1;
-					p = fgets(b, READBUF, f);
-					if (nobuf)
-						if ((fpos = ftello(f)) == -1)
-							warn("%s", name);
-					canjump = 0;
 				}
+
+				if (nobuf)
+					fseeko(f, fpos, SEEK_SET);
+				canjump = 1;
+				p = fgets(b, READBUF, f);
+				if (nobuf)
+					if ((fpos = ftello(f)) == -1)
+						warn("%s", name);
+				canjump = 0;
+
 				if (p == NULL || *b == '\0') {
 					if (ferror(f))
 						warn("%s", name);
 					eofline = fline;
 					eof = 1;
 					break;
-				} else {
-					if (!nobuf)
-						fputs(b, fbuf);
-					fwrite_all(&pos, sizeof pos, 1, find);
-					if (!fflag) {
-						oldpos = pos;
-						p = b;
-						while (*(p = endline(ttycols,
-								     p))
-						       != '\0') {
-							pos = oldpos + (p - b);
-							fwrite_all(&pos,
-								   sizeof pos,
-								   1, find);
-							fline++;
-							bline++;
-						}
-					}
-					fline++;
 				}
+
+				if (!nobuf)
+					fputs(b, fbuf);
+				fwrite_all(&pos, sizeof pos, 1, find);
+				if (!fflag) {
+					oldpos = pos;
+					p = b;
+					while (*(p = endline(ttycols,
+							     p))
+					       != '\0') {
+						pos = oldpos + (p - b);
+						fwrite_all(&pos,
+							   sizeof pos,
+							   1, find);
+						fline++;
+						bline++;
+					}
+				}
+				fline++;
 			} while (line > bline++);
 		} else {
 			/* eofline != 0 */
@@ -1057,7 +1054,9 @@ static void pgfile(FILE *f, const char *name)
 				skip(1);
 			}
 			continue;
-		} else if (eof) {
+		}
+
+		if (eof) {
 			/* We are not searching. */
 			line = bline;
 		} else if (*b != '\0') {
@@ -1379,10 +1378,9 @@ static void pgfile(FILE *f, const char *name)
 						my_sigset(SIGQUIT, oldquit);
 						my_sigset(SIGTERM, oldterm);
 						execl(sh, sh, "-c",
-						      cmd.cmdline + 1, NULL);
-						warn(_("failed to execute %s"), sh);
-						_exit(0177);
-						/* NOTREACHED */
+						      cmd.cmdline + 1, (char *)NULL);
+						errexec(sh);
+						break;
 					}
 					case -1:
 						mesg(_("fork() failed, "
@@ -1545,7 +1543,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	if (tcgetattr(STDOUT_FILENO, &otio) == 0) {
 		ontty = 1;
@@ -1571,7 +1569,7 @@ int main(int argc, char **argv)
 		}
 
 		if (!strcmp(argv[arg], "--version")) {
-		    printf(UTIL_LINUX_VERSION);
+		    print_version(EXIT_SUCCESS);
 		    return EXIT_SUCCESS;
 		}
 
@@ -1622,11 +1620,11 @@ int main(int argc, char **argv)
 			case 's':
 				sflag = 1;
 				break;
+
 			case 'h':
 				usage();
 			case 'V':
-				printf(UTIL_LINUX_VERSION);
-				return EXIT_SUCCESS;
+				print_version(EXIT_SUCCESS);
 			default:
 				invopt(&argv[arg][i]);
 			}
