@@ -103,6 +103,7 @@ int main(int argc, char *argv[])
 	size_t len, bufsiz = BUFSIZ;
 	FILE *fp = stdin;
 	int ch, rval = EXIT_SUCCESS;
+	uintmax_t line;
 
 	static const struct option longopts[] = {
 		{ "version",    no_argument,       NULL, 'V' },
@@ -144,6 +145,7 @@ int main(int argc, char *argv[])
 			filename = *argv++;
 		}
 
+		line = 0;
 		while (fgetws(buf, bufsiz, fp)) {
 			len = wcslen(buf);
 
@@ -168,9 +170,10 @@ int main(int argc, char *argv[])
 				buf[len--] = '\0';
 			reverse_str(buf, len);
 			fputws(buf, stdout);
+			line++;
 		}
 		if (ferror(fp)) {
-			warn("%s", filename);
+			warn("%s: %ju", filename, line);
 			rval = EXIT_FAILURE;
 		}
 		if (fp != stdin)
